@@ -1,4 +1,5 @@
-// +build !windows_v2
+//go:build windows
+// +build windows
 
 /*
    Copyright The containerd Authors.
@@ -21,7 +22,7 @@ package main
 import (
 	_ "github.com/containerd/containerd/diff/lcow"
 	_ "github.com/containerd/containerd/diff/windows"
+	_ "github.com/containerd/containerd/runtime/v2"
 	_ "github.com/containerd/containerd/snapshots/lcow"
 	_ "github.com/containerd/containerd/snapshots/windows"
-	_ "github.com/containerd/containerd/windows"
 )
