@@ -1,3 +1,4 @@
+//go:build windows
 // +build windows
 
 /*
@@ -20,15 +21,11 @@ package v2
 
 import (
 	"github.com/containerd/containerd/platforms"
-	ocispec "github.com/opencontainers/image-spec/specs-go/v1"
 )
 
-func supportedPlatforms() []ocispec.Platform {
-	return []ocispec.Platform{
-		platforms.DefaultSpec(),
-		{
-			OS:           "linux",
-			Architecture: "amd64",
-		},
+func defaultPlatforms() []string {
+	return []string{
+		platforms.DefaultString(),
+		"linux/amd64",
 	}
 }
